require_relative 'helpers'

module AteTracks
  module Structures
    def self.not_empty?(array)
      (array.is_a?(Array) && array.length > 0)
    end

    def self.populate(data, object)
      data.map { |elem| object.new(elem) }
    end

    class Response
      attr_accessor :content, :status, :errors, :notices, :logged_in, :api_version

      def initialize(data, type)
        class_type = Structures.const_get(type)
        if class_type
          @content = class_type.new(data[Helpers.underscore(type)])
          @status = data['status']
          @errors = data['errors']
          @notices = data['notices']
          @logged_in = data['logged_in']
          @api_version = data['api_version']
        end
      end
    end

    class PlayToken
      attr_accessor :play_token
      def initialize(data)
        return nil if data.nil?
        @play_token = data
      end
    end

    class Set
      attr_accessor :at_beginning, :at_end, :at_last_track, :skip_allowed, :track

      def initialize(data)
        return nil if data.nil?
        @at_beginning = data['at_beginning']
        @at_end = data['at_end']
        @at_last_track = data['at_last_track']
        @skip_allowed = data['skip_allowed']
        @track = Track.new(data['track'])
      end
    end

    class Track
      attr_accessor :track_annotation, :artist_details, :buy_link, :buy_icon, :buy_text, :track_file_stream_url,
                    :stream_source, :user_id, :full_length, :id, :name, :performer, :release_name, :url, :year,
                    :uid, :report_delay_s, :media_type

      def initialize(data)
        return nil if data.nil?
        @track_annotation = data['track_annotation']
        @artist_details = ArtistDetails.new(data['artist_details'])
        @buy_link = data['buy_link']
        @buy_icon = data['buy_icon']
        @buy_text = data['buy_text']
        @track_file_stream_url = data['track_file_stream_url']
        @stream_source = data['stream_source']
        @user_id = data['user_id']
        @full_length = data['full_length']
        @id = data['id']
        @name = data['name']
        @performer = data['performer']
        @release_name = data['release_name']
        @url = data['url']
        @year = data['year']
        @uid = data['uid']
        @report_delay_s = data['report_delay_s']

        unless @track_file_stream_url.nil?
          @media_type = (@track_file_stream_url == 'upload_v3') ? 'm4a' : 'mp3'
        end
      end
    end

    class ArtistDetails
      attr_accessor :twitter_username, :official_url, :wikipedia_url, :location, :years_active, :name, :bio_intro,
                    :bio_url, :image_url, :imgix_url, :url, :info_source

      def initialize(data)
        return nil if data.nil?
        @twitter_username = data['twitter_username']
        @official_url = data['official_url']
        @wikipedia_url = data['wikipedia_url']
        @location = data['location']
        @years_active = data['years_active']
        @name = data['name']
        @bio_intro = data['bio_intro']
        @bio_url = data['bio_url']
        @image_url = data['image_url']
        @imgix_url = data['imgix_url']
        @url = data['url']
        @info_source = InfoSource.new(data['info_source'])
      end
    end

    class InfoSource
      attr_accessor :source, :id, :bio_source

      def initialize(data)
        return nil if data.nil?
        @source = data['source']
        @id = data['id']
        @bio_source = data['bio_source']
      end
    end

    class MixSet
      attr_accessor :pagination, :mixes, :filters

      def initialize(data)
        return nil if data.nil?
        @pagination = Pagination.new(data['pagination'])
        @mixes = []
        if Structures::not_empty?(data['mixes'])
          @mixes = Structures::populate(data['mixes'], Mix)
        end
      end
    end

    class Mix
      attr_accessor :description_html, :likes_count, :certification, :user, :duration, :tracks_count, :nsfw,
                    :genres, :artists, :id, :published, :cover_urls, :description, :plays_count, :tag_list_cache,
                    :first_published_at, :first_published_at_timestamp, :is_promoted, :has_genres, :has_artists,
                    :name, :path, :web_path

      def initialize(data)
        return nil if data.nil?
        @description_html = data['description_html']
        @likes_count = data['likes_count']
        @certification = data['certification']
        @user = User.new(data['user'])
        @duration = data['duration']
        @tracks_count = data['tracks_count']
        @nsfw = data['nsfw']
        @genres = data['genres']
        @artists = data['artists']
        @id = data['id']
        @published = data['published']
        @cover_urls = CoverURLs.new(data['cover_urls'])
        @description = data['description']
        @plays_count = data['plays_count']
        @tag_list_cache = data['tag_list_cache']
        @first_published_at = data['first_published_at']
        @first_published_at_timestamp = data['first_published_at_timestamp']
        @is_promoted = data['is_promoted']
        @has_genres = data['has_genres']
        @has_artists = data['has_artists']
        @name = data['name']
        @path = data['path']
        @web_path = data['web_path']
      end
    end

    class User
      attr_accessor :id, :login, :path, :web_path, :avatar_urls, :location, :bio_html, :website, :country,
                    :member_since, :bio

      def initialize(data)
        return nil if data.nil?
        @id = data['id']
        @login = data['login']
        @path = data['path']
        @web_path = data['web_path']
        @avatar_urls = AvatarURLs.new(data['avatar_urls'])
        @location = data['location']
        @bio_html = data['bio_html']
        @website = data['website']
        @country = data['country']
        @member_since = data['member_since']
        @bio = data['bio']
      end
    end

    class UserMixes
      attr_accessor :mixes, :pagination, :smart_id, :smart_type, :path, :name, :web_path, :sort, :mix_set_id

      def initialize(data)
        return nil if data.nil?
        @mixes = []
        @pagination = Pagination.new(data['pagination'])
        @smart_id = data['smart_id']
        @smart_type = data['smart_type']
        @path = data['path']
        @name = data['name']
        @web_path = data['web_path']
        @sort = data['sort']
        @mix_set_id = data['mix_set_id']

        if Structures::not_empty?(data['mixes'])
          @mixes = Structures::populate(data['mixes'], Mix)
        end
      end
    end

    class SimilarMixes
      attr_accessor :mixes, :smart_id

      def initialize(data)
        return nil if data.nil?
        @mixes = []
        @smart_id = data['smart_id']

        if Structures::not_empty?(data['mixes'])
          @mixes = Structures::populate(data['mixes'], Mix)
        end
      end
    end

    class RelatedCollections
      attr_accessor :collections

      def initialize(data)
        return nil if data.nil?
        @collections = []

        if Structures::not_empty?(data['collections'])
          @collections = Structures::populate(data['@collections'], Collection)
        end
      end
    end

    class Pagination
      attr_accessor :current_page, :per_page, :offset_by, :next_page, :previous_page, :total_entries,
                    :total_pages, :next_page_path

      def initialize(data)
        return nil if data.nil?
        @current_page = data['current_page']
        @per_page = data['per_page']
        @offset_by = data['offset_by']
        @next_page = data['next_page']
        @previous_page = data['previous_page']
        @total_entries = data['total_entries']
        @total_pages = data['total_pages']
        @next_page_path = data['next_page_path']
      end
    end

    class Filter
      attr_accessor :color1, :color2, :name

      def initialize(data)
        return nil if data.nil?
        @color1 = data['color1']
        @color2 = data['color2']
        @name = data['name']
      end
    end

    class Collection
      attr_accessor :tag_list_cache, :cover_urls, :duration, :user, :id, :smart_id, :name, :slug, :mixes_count,
                    :web_path, :description, :updated_at

      def initialize(data)
        return nil if data.nil?
        @tag_list_cache = data['tag_list_cache']
        @cover_urls = data['cover_urls']
        @duration = data['duration']
        @user = data['user']
        @id = data['id']
        @smart_id = data['smart_id']
        @name = data['name']
        @slug = data['slug']
        @mixes_count = data['mixes_count']
        @web_path = data['web_path']
        @description = data['description']
        @updated_at = data['updated_at']
      end
    end

    class AvatarURLs
      attr_accessor :original, :original_imgix_url, :static_cropped_imgix_url, :cropped_imgix_url, :sq56, :sq72,
                    :sq100, :max200, :max250w, :discourse, :animated

      def initialize(data)
        return nil if data.nil?
        @original = data['original']
        @original_imgix_url = data['original_imgix_url']
        @static_cropped_imgix_url = data['static_cropped_imgix_url']
        @cropped_imgix_url = data['cropped_imgix_url']
        @sq56 = data['sq56']
        @sq72 = data['sq72']
        @sq100 = data['sq100']
        @max200 = data['max200']
        @max250w = data['max250w']
        @discourse = data['discourse']
        @animated = data['animated']
      end
    end

    class CoverURLs
      attr_accessor :original, :original_imgix_url, :static_cropped_imgix_url, :cropped_imgix_url, :sq56, :sq72,
                    :sq100, :sq133, :max133w, :max200, :sq250, :sq500, :max1024

      def initialize(data)
        return nil if data.nil?
        @original = data['original']
        @original_imgix_url = data['original_imgix_url']
        @static_cropped_imgix_url = data['static_cropped_imgix_url']
        @cropped_imgix_url = data['cropped_imgix_url']
        @sq56 = data['sq56']
        @sq72 = data['sq72']
        @sq100 = data['sq100']
        @sq133 = data['sq133']
        @max133w = data['max133w']
        @max200 = data['max200']
        @sq250 = data['sq250']
        @sq500 = data['sq500']
        @max1024 = data['max1024']
      end
    end
  end
end